@@ -32,6 +32,7 @@ class PerturbationTheoryCorrections:
                  degenerate_transformation=None,
                  degenerate_energies=None,
                  degenerate_hamiltonians=None,
+                 nondeg_hamiltonian_precision=3,
                  logger=None
                  ):
         """
@@ -63,6 +64,7 @@ class PerturbationTheoryCorrections:
         self.degenerate_energies = degenerate_energies
         self.degenerate_hamiltonians = degenerate_hamiltonians
         self.logger = logger
+        self.nondeg_hamiltonian_precision = nondeg_hamiltonian_precision
 
     @classmethod
     def from_dicts(cls,
@@ -374,7 +376,7 @@ class PerturbationTheoryCorrections:
                 for x in self.operator_representation(subhams, logger_symbol="H", logger_conversion=UnitsData.convert("Hartrees", "Wavenumbers"))
             ]
         return H_nd
-    def get_degenerate_rotation(self, deg_group, hams):
+    def get_degenerate_rotation(self, deg_group, hams, zero_point_energy=None):
         """
 
         :param deg_group:
@@ -404,11 +406,19 @@ class PerturbationTheoryCorrections:
         # for h in H_nd[1:]:
         #     np.fill_diagonal(h, 0.)
         H_nd_corrs = subdegs.get_transformed_Hamiltonians(hams, deg_group=None)
+        group_inds = self.states.find(deg_group)
+        # zero_order_engs = self.energy_corrs[group_inds, 0]
+        # raise Exception(
+        #     (H_nd_corrs[0]-np.diag(np.full(len(group_inds), self.energy_corrs[0, 0])))*UnitsData.convert("Hartrees", "Wavenumbers"),
+        #     (zero_order_engs-self.energy_corrs[0, 0])*UnitsData.convert("Hartrees", "Wavenumbers")
+        # )
+        # np.fill_diagonal(H_nd_corrs[0], zero_order_engs)
+
         # import McUtils.Plots as plt
         # plt.TensorPlot(np.array(H_nd)).show()
         H_nd = np.sum(H_nd_corrs, axis=0)
         if np.sum(H_nd) == 0:
-            raise Exception(subdegs.wfn_corrections)
+            raise ValueError("No corrections from ", subdegs.wfn_corrections)
         #     raise Exception(deg_group.excitations,
         #                     self.states.take_states(deg_group).excitations,
         #                     # self.coupled_states.take_states(deg_group).excitations
@@ -416,11 +426,14 @@ class PerturbationTheoryCorrections:
         # overlaps = np.sum(subdegs.get_overlap_matrices(), axis=0)
 
         with logger.block(tag="non-degenerate Hamiltonian"):
-            logger.log_print(
-                str(
-                    np.round(H_nd * UnitsData.convert("Hartrees", "Wavenumbers")).astype(int)
-                ).splitlines()
-            )
+            if zero_point_energy is None:
+                zero_point_energy = self.energies[0]
+            with np.printoptions(precision=self.nondeg_hamiltonian_precision, suppress=True):
+                logger.log_print(
+                    str(
+                        (H_nd - np.diag(np.full(len(group_inds), zero_point_energy))) * UnitsData.convert("Hartrees", "Wavenumbers")
+                    ).splitlines()
+                )
 
         deg_engs, deg_transf = np.linalg.eigh(H_nd)
 
@@ -467,7 +480,7 @@ class PerturbationTheoryCorrections:
 
         return H_nd_corrs, deg_engs, deg_transf
 
-    def get_degenerate_transformation(self, group, hams, gaussian_resonance_handling=False):
+    def get_degenerate_transformation(self, group, hams, gaussian_resonance_handling=False, zero_point_energy=None):
         # this will be built from a series of block-diagonal matrices
         # so we store the relevant values and indices to compose the SparseArray
 
@@ -490,7 +503,7 @@ class PerturbationTheoryCorrections:
                 gaussian_resonance_handling and np.max(np.sum(group.excitations, axis=1)) > 2):
             H_nd = deg_engs = deg_rot = None
         elif len(deg_inds) > 1:
-            H_nd, deg_engs, deg_rot = self.get_degenerate_rotation(group, hams)
+            H_nd, deg_engs, deg_rot = self.get_degenerate_rotation(group, hams, zero_point_energy=zero_point_energy)
         else:
             H_nd = deg_engs = deg_rot = None
             # raise NotImplementedError("Not sure what to do when no states in degeneracy spec are in total space")
