@@ -5,15 +5,15 @@ A little package of utilities for setting up/running VPT jobs
 import numpy as np, sys, os, itertools, scipy
 
 from McUtils.Data import UnitsData, AtomData
-from McUtils.Scaffolding import ParameterManager, Checkpointer
+from McUtils.Scaffolding import ParameterManager
 from McUtils.Zachary import FiniteDifferenceDerivative
+from McUtils.Extensions import ModuleLoader
 
-from ..BasisReps import BasisStateSpace, HarmonicOscillatorProductBasis
+from ..BasisReps import BasisStateSpace, HarmonicOscillatorProductBasis, BasisStateSpaceFilter
 from ..Molecools import Molecule
 
 from .DegeneracySpecs import DegeneracySpec
 from .Hamiltonian import PerturbationTheoryHamiltonian
-from .StateFilters import PerturbationTheoryStateSpaceFilter
 
 __all__ = [
     "VPTRunner",
@@ -25,7 +25,7 @@ __all__ = [
     "VPTSolverOptions"
 ]
 
-__reload_hook__ = ["..BasisRepss", "..Molecools", ".DegeneracySpecs", ".Hamiltonian", ".StateFilters"]
+__reload_hook__ = ["..BasisReps", "..Molecools", ".DegeneracySpecs", ".Hamiltonian", ".StateFilters"]
 
 class VPTSystem:
     """
@@ -36,6 +36,7 @@ class VPTSystem:
     __props__ = (
         "internals",
         "modes",
+        "mode_selection",
         "potential_derivatives",
         "potential_function",
         "order",
@@ -48,6 +49,7 @@ class VPTSystem:
                  internals=None,
                  dummy_atoms=None,
                  modes=None,
+                 mode_selection=None,
                  potential_derivatives=None,
                  potential_function=None,
                  order=2,
@@ -86,6 +88,7 @@ class VPTSystem:
         self.mol = mol
         if modes is not None:
             self.mol.normal_modes.modes = modes
+        self.mode_selection = mode_selection
         if potential_derivatives is not None:
             self.mol.potential_derivatives = potential_derivatives
         elif potential_function is not None:
@@ -112,7 +115,10 @@ class VPTSystem:
         :return:
         :rtype:
         """
-        return len(self.mol.normal_modes.modes.freqs)
+        if self.mode_selection is not None:
+            return len(self.mode_selection)
+        else:
+            return len(self.mol.normal_modes.modes.freqs)
 
     def get_potential_derivatives(self, potential_function, order=2, **fd_opts):
         """
@@ -288,11 +294,11 @@ class VPTStateSpace:
         # else:
         #     raise NotImplementedError("don't know what to do with degeneracy spec {}".format(degeneracy_specs))
 
-    def filter_generator(self, target_property, order=2):
+    def filter_generator(self, target_property, order=2, postfilters=None):
         def filter(states):
-            return self.get_state_space_filter(states, target=target_property, order=order)
+            return self.get_state_space_filter(states, target=target_property, order=order, postfilters=postfilters)
         return filter
-    def get_filter(self, target_property, order=2):
+    def get_filter(self, target_property, order=2, postfilters=None):
         """
         Obtains a state space filter for the given target property
         using the states we want to get corrections for
@@ -306,10 +312,11 @@ class VPTStateSpace:
         """
         return self.get_state_space_filter(self.state_list,
                                            target=target_property,
-                                           order=order
+                                           order=order,
+                                           postfilters=postfilters
                                            )
     @classmethod
-    def get_state_space_filter(cls, states, n_modes=None, order=2, target='wavefunctions'):
+    def get_state_space_filter(cls, states, n_modes=None, order=2, target='wavefunctions', postfilters=None):
         """
         Gets `state_space_filters` for the input `states` targeting some property
 
@@ -334,7 +341,7 @@ class VPTStateSpace:
         if target == 'wavefunctions':
             return None
         elif target == 'intensities':
-            return PerturbationTheoryStateSpaceFilter.from_property_rules(
+            return BasisStateSpaceFilter.from_property_rules(
                 cls.get_state_list_from_quanta(0, n_modes),
                 states,
                 [
@@ -343,21 +350,23 @@ class VPTStateSpace:
                 [
                     HarmonicOscillatorProductBasis(n_modes).selection_rules(*["x"]*i) for i in range(1, order+2)
                 ],
-                order=order
+                order=order,
+                postfilters=postfilters
             )
         elif target == 'frequencies':
             # return {
             #     (1, 1): ([],),
             #     (2, 0): (None, [[0]])
             # }
-            return PerturbationTheoryStateSpaceFilter.from_property_rules(
+            return BasisStateSpaceFilter.from_property_rules(
                 cls.get_state_list_from_quanta(0, n_modes),
                 states,
                 [
                     HarmonicOscillatorProductBasis(n_modes).selection_rules(*["x"] * i) for i in range(3, order + 3)
                 ],
                 None,
-                order=order
+                order=order,
+                postfilters=postfilters
                 # [
                 #     # (),
                 #     # (),
@@ -411,7 +420,8 @@ class VPTHamiltonianOptions:
          "freq_tolerance",
          "g_derivative_threshold",
          "gmatrix_tolerance",
-         'use_cartesian_kinetic_energy'
+         'use_cartesian_kinetic_energy',
+         'operator_coefficient_threshold'
     )
 
     def __init__(self,
@@ -451,7 +461,8 @@ class VPTHamiltonianOptions:
                  g_derivative_threshold=None,
                  gmatrix_tolerance=None,
                  use_internal_modes=None,
-                 use_cartesian_kinetic_energy=None
+                 use_cartesian_kinetic_energy=None,
+                 operator_coefficient_threshold=None
                  ):
         """
         :param mode_selection: the subset of normal modes to do perturbation theory on
@@ -544,7 +555,8 @@ class VPTHamiltonianOptions:
             freq_tolerance=freq_tolerance,
             g_derivative_threshold=g_derivative_threshold,
             gmatrix_tolerance=gmatrix_tolerance,
-            use_cartesian_kinetic_energy=use_cartesian_kinetic_energy
+            use_cartesian_kinetic_energy=use_cartesian_kinetic_energy,
+            operator_coefficient_threshold=operator_coefficient_threshold
         )
 
         real_opts = {}
@@ -953,6 +965,7 @@ class VPTRunner:
                system,
                states,
                target_property=None,
+               basis_filters=None,
                corrected_fundamental_frequencies=None,
                **opts
                ):
@@ -989,14 +1002,14 @@ class VPTRunner:
             )
 
         order = 2 if 'order' not in opts.keys() else opts['order']
-        if target_property is None and order == 2:
+        if target_property is None:
             target_property = 'intensities'
         if target_property is not None and 'state_space_filters' not in opts:
             if 'expansion_order' in opts.keys():
                 expansion_order = opts['expansion_order']
             else:
                 expansion_order = order
-            par.ops['state_space_filters'] = states.filter_generator(target_property, order=order)
+            par.ops['state_space_filters'] = states.filter_generator(target_property, order=order, postfilters=basis_filters)
 
             # print(par.ops['state_space_filters'])
 
@@ -1537,6 +1550,7 @@ class AnneInputHelpers:
                      potential_files=('cub.dat', 'quart.dat', 'quintic.dat', 'sextic.dat'),
                      dipole_files=('lin_dip.dat', 'quad_dip.dat', "cub_dip.dat", "quart_dip.dat", 'quintic_dip.dat'),
                      coordinate_transformation=None,
+                     coordinate_transformation_file='coordinate_transformation.py',
                      results_file=None,#'output.hdf5',
                      order=None,
                      expansion_order=None,
@@ -1592,6 +1606,16 @@ class AnneInputHelpers:
                 else:
                     zmat = None
             sorting = cls.standard_sorting(zmat)  # we need to re-sort our internal coordinates
+            if coordinate_transformation is None:
+                if os.path.isfile(coordinate_transformation_file):
+                    tf_mod = ModuleLoader().load(coordinate_transformation_file)
+                    if not hasattr(tf_mod, 'transformation') or not hasattr(tf_mod, 'inverse'):
+                        raise ValueError("Coordinate transformation module {} needs both '{}' and '{}' methods".format(
+                            coordinate_transformation_file,
+                            'transformation',
+                            'inverse'
+                        ))
+                    coordinate_transformation = [tf_mod.transformation, tf_mod.inverse]
             if coordinate_transformation is not None:
                 if callable(coordinate_transformation) or len(coordinate_transformation) != 2:
                     raise ValueError("need both a coordinate transformation and inverse passed like `[tf, inv]`")
@@ -1647,7 +1671,7 @@ class AnneInputHelpers:
                 if 'kinetic' not in expansion_order:
                     expansion_order['kinetic'] = 2
                 if dipole_terms is not None and 'dipole' not in expansion_order:
-                    expansion_order['dipole'] = len(dipole_terms) - 1
+                    expansion_order['dipole'] = len(dipole_terms[0]) - 2
             if order is None:
                 if isinstance(expansion_order, int):
                     if expansion_order > 2:
@@ -1656,7 +1680,6 @@ class AnneInputHelpers:
                         order = 2
                 else:
                     order = expansion_order['potential']
-            # raise Exception([a.shape for a in dipole_terms])
 
             res = runner(
                 [atoms, coords, dict(masses=masses)],
