@@ -8,14 +8,13 @@ from McUtils.Parallelizers import Parallelizer, SerialNonParallelizer
 from McUtils.Data import UnitsData
 from McUtils.Combinatorics import LatticePathGenerator
 
-from ..BasisReps import Representation, BasisStateSpace, BasisMultiStateSpace, SelectionRuleStateSpace, BraKetSpace
+from ..BasisReps import Representation, BasisStateSpace, BasisMultiStateSpace, SelectionRuleStateSpace, BasisStateSpaceFilter
 
-from .StateFilters import PerturbationTheoryStateSpaceFilter
 from .DegeneracySpecs import DegenerateMultiStateSpace, DegeneracySpec
 from .Common import *
 from .Corrections import *
 
-__reload_hook__ = [ "..BasisReps", ".DegeneracySpecs", ".Corrections", ".StateFilters", ".Common" ]
+__reload_hook__ = [ "..BasisReps", ".DegeneracySpecs", ".Corrections", ".Common" ]
 
 __all__ = [
     "PerturbationTheorySolver"
@@ -101,10 +100,10 @@ class PerturbationTheorySolver:
 
         if isinstance(state_space_filters, (types.FunctionType, types.MethodType, types.LambdaType)):
             self.state_space_filter_generator = state_space_filters
-            self.state_space_filters = PerturbationTheoryStateSpaceFilter.from_data(states, self.state_space_filter_generator(states))
+            self.state_space_filters = BasisStateSpaceFilter.from_data(states, self.state_space_filter_generator(states))
         else:
             self.state_space_filter_generator = None
-            self.state_space_filters = PerturbationTheoryStateSpaceFilter.from_data(states, state_space_filters)
+            self.state_space_filters = BasisStateSpaceFilter.from_data(states, state_space_filters)
         self.target_property_rules=target_property_rules
 
         self.logger = logger
@@ -128,6 +127,8 @@ class PerturbationTheorySolver:
                     'wfc_threshold':'auto' if handle_strong_couplings is True else handle_strong_couplings,
                     'iterations':order//2
                 })
+        elif hasattr(self.degeneracy_spec, 'wfc_threshold') and self.degeneracy_spec.iterations is None:
+            self.degeneracy_spec.iterations = order//2
         self.handle_strong_couplings = hasattr(self.degeneracy_spec, 'wfc_threshold')
         self.nondeg_hamiltonian_precision=nondeg_hamiltonian_precision
 
@@ -689,7 +690,7 @@ class PerturbationTheorySolver:
                 for p,cs in zip(self.perts[1:], self.coupled_states):
                     existing_spaces[p] = ({None:cs}, cs)
                 if self.state_space_filter_generator is not None:
-                    filters = PerturbationTheoryStateSpaceFilter.from_data(new_targets, self.state_space_filter_generator(new_targets))
+                    filters = BasisStateSpaceFilter.from_data(new_targets, self.state_space_filter_generator(new_targets))
                     # raise Exception(filters[(1, 1)].prefilters)
                 else:
                     filters = None
@@ -928,14 +929,14 @@ class PerturbationTheorySolver:
                     and self._could_be_rules(test[1])
             )
         )
-    def _apply_transformation_with_filters(self, a, b, filter_space:PerturbationTheoryStateSpaceFilter, **opts):
+    def _apply_transformation_with_filters(self, a, b, filter_space:BasisStateSpaceFilter, **opts):
 
         if filter_space is not None:
             prefilters = filter_space.prefilters
-            postfilter = filter_space.postfilter
+            postfilters = filter_space.postfilters
         else:
             prefilters = None
-            postfilter = None
+            postfilters = None
 
         if prefilters is not None:
             # this means we are able to filter _before_ we apply the selection rules
@@ -974,12 +975,12 @@ class PerturbationTheorySolver:
             new = a.get_transformed_space(b, **opts)
 
         if new is not None: # possible to have None if we had to do no work
-            if postfilter is not None:
-                if not isinstance(postfilter, (BasisStateSpace, BasisMultiStateSpace)):
-                    postfilter = BasisStateSpace(b.basis, postfilter)
-                if isinstance(postfilter, SelectionRuleStateSpace):
-                    postfilter = postfilter.to_single().take_unique()
-                new = new.take_states(postfilter)
+            if postfilters is not None:
+                for filter in postfilters:
+                    if isinstance(new, BasisMultiStateSpace):
+                        new = new.map(filter.apply)
+                    else:
+                        new = filter.apply(new)
 
         return new
 
@@ -1020,6 +1021,7 @@ class PerturbationTheorySolver:
             raise NotImplementedError("we shouldn't be here")
             new = a * b
         elif isinstance(a, self.ProjectionOperatorWrapper):
+            raise NotImplementedError("we shouldn't be here")
             # uhhh...
             new = a.get_transformed_space(b, parallelizer=self.parallelizer, logger=logger)
         elif isinstance(a, (self.ProjectedOperator, Representation)):
@@ -1099,7 +1101,7 @@ class PerturbationTheorySolver:
                             )
 
                             if new_new is None:
-                                new = b
+                                new = b.to_single().take_unique()
                             else:
                                 # next we add the new stuff to the cache
                                 cur = cur.union(new_new)
@@ -1110,9 +1112,9 @@ class PerturbationTheorySolver:
                                 if ret_space:
                                     new = existing.union(new_new).to_single(track_excitations=not self.memory_constrained).take_unique()
                                 else:
-                                    new = b
+                                    new = b.to_single().take_unique()
                         else:
-                            new = b
+                            new = b.to_single().take_unique()
 
                     else:
                         # means we already calculated everything
@@ -1384,6 +1386,10 @@ class PerturbationTheorySolver:
 
     def construct_strong_coupling_spaces(self, sc, corrs, states, threshold):
         sc = corrs.collapse_strong_couplings(sc)
+        # self.logger.log_print(
+        #     corrs.states.excitations,
+        #     message_prepper=lambda a: str(np.array(a)).splitlines()
+        # )
         with self.logger.block(tag='unpruned spaces states', log_level=self.logger.LogLevel.Never):
             for s in sc.values():
                 self.logger.log_print(
@@ -1476,6 +1482,8 @@ class PerturbationTheorySolver:
         :rtype:
         """
 
+        # print("????", states.excitations)
+
         with checkpointer:
 
             all_energies = np.zeros((len(states), order + 1))
@@ -1650,7 +1658,7 @@ class PerturbationTheorySolver:
                                     N,
                                     checkpointer,
                                     logger,
-                                    None,
+                                    degenerate_states,
                                     handle_strong_couplings=True,
                                     non_zero_cutoff=non_zero_cutoff
                                 )
